from datetime import date
import sqlite3
# from PIL import Image

class Book:
    def __init__(self, ISBN: str, title: str, cover: bytes, quantity: int):
        self.__ISBN = ISBN
        self.__title = title
        self.__cover = cover
        # status not loaned ketika tidak ada yang pinjam 
        # asumsi suatu buku ada beberapa jumlahnya
        self.__status = "not loaned"
        # jumlah awalnya jumlah buku dalam perpustakaan
        # dikurangi sesuai peminjaman
        self.__quantity = quantity

    def getISBN(self) -> str:
        return self.__ISBN

    def getTitle(self) -> str:
        return self.__title

    def getCover(self) -> bytes:
        return self.__cover
    
    def getStatus(self) -> str:
        return self.__status

    def getQuantity(self) -> int:
        return self.__quantity

    def setISBN(self, ISBN):
        self.__ISBN = ISBN

    def setTitle(self, title):
        self.__title = title

    def setCover(self, cover):
        self.__cover = cover

    def setStatus(self, status):
        self.__status = status

    def setQuantity(self, quantity):
        # Jumlah jadi 0 -> status "not loaned"
        if (self.__quantity == 0 and quantity > 0):
            self.__quantity = quantity
            self.setStatus("loaned")
        # Jumlah sebelum diubah 0 dan mau diubah jadi ada
        elif (self.__quantity > 0 and quantity == 0):
            self.__quantity = quantity
            self.setStatus("not loaned")

class BookController:
    def __init__(self, connection):
        self.connection = connection
        self.create_table()

    def create_table(self):
        cursor = self.connection.cursor()
        cursor.execute('''
            CREATE TABLE IF NOT EXISTS Book (
                ISBN TEXT PRIMARY KEY,
                Title TEXT NOT NULL,
                Cover BLOB,
                Status TEXT NOT NULL,
                Quantity INTEGER NOT NULL
            )
        ''')
        self.connection.commit()

    def addBook(self, buku):
        cursor = self.connection.cursor()
        cursor.execute('''
            INSERT INTO Book (ISBN, Title, Cover, Status, Quantity)
            VALUES (?, ?, ?, ?, ?)
        ''', (buku.getISBN(), buku.getTitle(), buku.getCover(), buku.getStatus(), buku.getQuantity()))
        self.connection.commit()
    
    def addBookByFields(self, isbn, title, cover, quantity):
        cursor = self.connection.cursor()
        cursor.execute('''
            INSERT INTO Book (ISBN, Title, Cover, Status, Quantity)
            VALUES (?, ?, ?, ?, ?)
        ''', (isbn, title, cover, "not loaned", quantity))
        self.connection.commit()

    def displayBook(self):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Book
        ''')
        return cursor.fetchall()

    def removeBook(self, ISBN):
        cursor = self.connection.cursor()

        # Cari Status
        cursor.execute('''
            SELECT Status FROM Book WHERE ISBN = ?
        ''', (ISBN,))
        bookStatus = cursor.fetchone()[0]

        # Hapus jika tidak ada yang meminjam
        if (bookStatus == "not loaned"):
            cursor.execute('''
                DELETE FROM Book WHERE ISBN = ?
            ''', (ISBN,))

        self.connection.commit()

    def editBook(self, ISBN, Title, Cover, Status, Quantity):
        cursor = self.connection.cursor()
        cursor.execute('''
            UPDATE Book SET Title = ?, Cover = ?, Status = ?, Quantity = ? WHERE ISBN = ?
        ''', (Title, Cover, Status, Quantity, ISBN))
        self.connection.commit()

    def getBook(self, ISBN):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Book WHERE ISBN = ?
        ''', (ISBN,))
        return cursor.fetchone()
    
    def searchBook(self, input):
        wildc_input = "%"+input+"%"
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Book WHERE ISBN LIKE ? OR Title LIKE ?
        ''', (wildc_input,wildc_input,))
        return cursor.fetchall()