import flet as ft
import sqlite3
import time
from datetime import date
from gui.widgets import NavBar, ConfirmDialog, SearchBar, ErrorDialog
from loan.Loan import LoanController
from book.Book import BookController
from member.Member import MemberController

class LoanContainer(ft.Container):
    def __init__(self, loanlist: ft.GridView, id: int, book:str, member:int, borrowDate:date, returnDate:date):
        super().__init__()
        self.loanlist = loanlist
        self.id = id
        self.book = book
        self.member = member
        self.borrowDate = borrowDate
        self.returnDate = returnDate

        self.editButton = ft.IconButton(
            icon=ft.icons.EDIT,
            bgcolor=ft.colors.DEEP_PURPLE_ACCENT,
            icon_color=ft.colors.WHITE,
            tooltip="Edit loan",
            on_click=self.edit_loan
        )
        self.returnBookButton = ft.IconButton(
            icon=ft.icons.KEYBOARD_RETURN,
            bgcolor=ft.colors.BLUE_ACCENT,
            icon_color=ft.colors.WHITE,
            tooltip="Return book",
            on_click=self.return_book
        )

        self.bgcolor = ft.colors.INDIGO_900
        self.padding = 10
        self.content = ft.Row(
            vertical_alignment=ft.CrossAxisAlignment.START,
            spacing=20,
            controls=[
                ft.Text(str(id),),
                ft.Column(
                    expand=3,
                    controls=[
                        ft.Text("Book: "+book),
                        ft.Text("Member: "+str(member)),
                        ft.Text("Borrowed: "+str(borrowDate)),
                        ft.Text("Return by: "+str(returnDate))
                    ]
                ),
                ft.Column(
                    controls=[
                        self.editButton, self.returnBookButton
                    ]
                )
            ]
        )

    def edit_loan(self, e: ft.TapEvent):
        dlg = LoanEdit(self, self.loanlist.page)
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            if dlg.newReturn.value < dlg.newBorrow.value:
                err = ErrorDialog(self.page, "Insert failed. Return date cannot be before borrow date.")
                err.open_dlg(e)
            else:
                loan_controller = LoanController(sqlite3.connect('src/db/library.db'))
                loan_controller.editLoan(self.id, dlg.newBorrow.value.date(), dlg.newReturn.value.date())
                self.loanlist.update_data()
    
    def return_book(self, e: ft.TapEvent):
        dlg = ConfirmDialog(self.loanlist.page, "return book")
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            loan_controller = LoanController(sqlite3.connect('src/db/library.db'))
            loan_controller.returnBook(self.id)
            self.loanlist.update_data()

class LoanEdit():
    def __init__(self, loan: ft.Container, page: ft.Page):
        self.loan = loan
        self.page = page
        self.doAction = False
        self.newBorrow = ft.DatePicker(value=self.loan.borrowDate, current_date=self.loan.borrowDate, field_hint_text=self.loan.borrowDate)
        self.newReturn = ft.DatePicker(value=self.loan.returnDate, current_date=self.loan.returnDate, field_hint_text=self.loan.returnDate)
        self.page.overlay.append(self.newBorrow)
        self.page.overlay.append(self.newReturn)

        self.newBorrowButton = ft.TextButton(
            text="Edit Borrow Date",
            icon=ft.icons.CALENDAR_MONTH,
            on_click=lambda _: self.newBorrow.pick_date(),
        )
        self.newReturnButton = ft.TextButton(
            text="Edit Return Date",
            icon=ft.icons.CALENDAR_MONTH,
            on_click=lambda _: self.newReturn.pick_date(),
        )

        self.dlg = ft.AlertDialog(
            modal=False,
            title=ft.Text("Edit Loan"),
            content=ft.Column(
                controls=[
                    ft.Text("ID: "+str(self.loan.id)),
                    ft.Text("Book ISBN: "+self.loan.book),
                    ft.Text("Member ID: "+str(self.loan.member)),
                    ft.Row(
                        controls=[self.newBorrowButton, self.newReturnButton]
                    )
                ]
            ),
            actions=[
                ft.TextButton("Cancel", on_click=self.close_dlg),
                ft.TextButton("Save", on_click=self.close_dlg_do),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            bgcolor=ft.colors.BLACK
        )

    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    
    def close_dlg_do(self, e: ft.TapEvent):
        self.doAction = True
        self.dlg.open = False
        self.page.update()
    
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()

class LoanAdd():
    def __init__(self, page: ft.Page):
        self.page = page
        self.doAction = False

        self.newBook = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250
        )
        self.newMember = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250
        )
        self.newBorrow = ft.DatePicker()
        self.newReturn = ft.DatePicker()
        self.page.overlay.append(self.newBorrow)
        self.page.overlay.append(self.newReturn)

        self.newBookRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Book ISBN: "), self.newBook
            ]
        )
        self.newMemberRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Member ID: "), self.newMember
            ]
        )
        self.newBorrowButton = ft.TextButton(
            text="Edit Borrow Date",
            icon=ft.icons.CALENDAR_MONTH,
            on_click=lambda _: self.newBorrow.pick_date(),
        )
        self.newReturnButton = ft.TextButton(
            text="Edit Return Date",
            icon=ft.icons.CALENDAR_MONTH,
            on_click=lambda _: self.newReturn.pick_date(),
        )

        self.dlg = ft.AlertDialog(
            modal=False,
            title=ft.Text("New Loan"),
            content=ft.Column(
                controls=[
                    self.newBookRow,
                    self.newMemberRow,
                    ft.Row(
                        controls=[self.newBorrowButton, self.newReturnButton]
                    )
                ]
            ),
            actions=[
                ft.TextButton("Cancel", on_click=self.close_dlg),
                ft.TextButton("Add", on_click=self.close_dlg_do),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            bgcolor=ft.colors.BLACK
        )

    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    
    def close_dlg_do(self, e: ft.TapEvent):
        self.doAction = True
        self.dlg.open = False
        self.page.update()
    
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()

class LoanList(ft.GridView):
    def __init__(self):
        super().__init__()
        self.child_aspect_ratio = 2.3
        self.runs_count = 3
    def update_data(self):
        loan_controller = LoanController(sqlite3.connect('src/db/library.db'))
        self.spacing = 10
        self.controls = []
        for l in loan_controller.displayLoan():
            lc = LoanContainer(self, l[0], l[1], l[2], l[3], l[4])
            self.controls.append(lc)
        self.update()

    def filter_data(self, input):
        loan_controller = LoanController(sqlite3.connect('src/db/library.db'))
        self.spacing = 10
        self.controls = []
        for l in loan_controller.searchLoan(input):
            lc = LoanContainer(self, l[0], l[1], l[2], l[3], l[4])
            self.controls.append(lc)
        self.update()

class LoanView(ft.View):
    def __init__(self, page:ft.Page):
        super().__init__()
        self.page = page
        self.route = "/loans"
        self.scroll = True
        self.loans = LoanList()
        self.search = SearchBar(self.loans)
        self.add = ft.ElevatedButton(text="New", icon=ft.icons.ADD, bgcolor=ft.colors.GREEN, color=ft.colors.WHITE, on_click=self.add_loan)
        self.controls=[
            ft.Column(
                horizontal_alignment=ft.CrossAxisAlignment.CENTER,
                expand=True,
                controls=[
                    NavBar(page),
                    ft.Text("LOAN LIST", weight=ft.FontWeight.BOLD, size=22),
                    ft.Row(
                        alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
                        controls=[
                            self.search, self.add
                        ]
                    ),
                    self.loans
                ]
            )
        ]
    
    def add_loan(self, e: ft.TapEvent):
        dlg = LoanAdd(self.page)
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            loan_controller = LoanController(sqlite3.connect('src/db/library.db'))
            book_controller = BookController(sqlite3.connect('src/db/library.db'))
            member_controller = MemberController(sqlite3.connect('src/db/library.db'))
            if dlg.newBook.value == None or dlg.newMember.value == None or dlg.newBorrow.value == None or dlg.newReturn.value == None:
                err = ErrorDialog(self.page, "Insert failed. All fields must be filled.")
                err.open_dlg(e)
            elif dlg.newReturn.value < dlg.newBorrow.value:
                err = ErrorDialog(self.page, "Insert failed. Return date cannot be before borrow date.")
                err.open_dlg(e)
            elif book_controller.getBook(dlg.newBook.value) == None:
                err = ErrorDialog(self.page, "Insert failed. Book does not exist.")
                err.open_dlg(e)
            elif book_controller.getBook(dlg.newBook.value)[4] < 1:
                err = ErrorDialog(self.page, "Insert failed. There are no books available to loan.")
                err.open_dlg(e)
            elif member_controller.getMember(dlg.newMember.value) == None:
                err = ErrorDialog(self.page, "Insert failed. Member does not exist.")
                err.open_dlg(e)
            elif member_controller.getMember(dlg.newMember.value)[6] >= 3:
                err = ErrorDialog(self.page, "Insert failed. Member is already loaning three books.")
                err.open_dlg(e)
            elif member_controller.getMember(dlg.newMember.value)[5] == "not active":
                err = ErrorDialog(self.page, "Insert failed. Member is not active.")
                err.open_dlg(e)
            else:
                loan_controller.addLoanByFields(dlg.newBook.value, int(dlg.newMember.value), dlg.newBorrow.value.date(), dlg.newReturn.value.date())
                self.loans.update_data()