import flet as ft
import time
import sqlite3
from gui.widgets import NavBar, ConfirmDialog, SearchBar, ErrorDialog
from member.Member import MemberController

class MemberContainer(ft.Container):
    def __init__(self, memberlist: ft.GridView, id: int, name: str, address: str, email: str, phone:str, status: str, booksBorrowed: int):
        super().__init__()
        self.memberlist = memberlist
        self.id = id
        self.name = name
        self.address = address
        self.email = email
        self.phone = phone
        self.status = status
        self.booksBorrowed = booksBorrowed

        self.editButton = ft.IconButton(
            icon=ft.icons.EDIT,
            bgcolor=ft.colors.DEEP_PURPLE_ACCENT,
            icon_color=ft.colors.WHITE,
            tooltip="Edit member",
            on_click=self.edit_member
        )
        self.deleteButton = ft.IconButton(
            icon=ft.icons.DELETE,
            bgcolor=ft.colors.RED_ACCENT,
            icon_color=ft.colors.WHITE,
            tooltip="Delete member",
            on_click=self.delete_member
        )

        self.bgcolor = ft.colors.INDIGO_900
        self.padding = 10
        self.content = ft.Row(
            spacing=20,
            controls=[
                ft.Text(str(id)),
                ft.Column(
                    expand=3,
                    controls=[
                        ft.Text("Name: "+name),
                        ft.Text("Address: "+address),
                        ft.Text("Email: "+email),
                        ft.Text("Phone Number: "+phone),
                        ft.Text("Status: "+status),
                        ft.Text("Books borrowed: "+str(booksBorrowed))
                    ]
                ),
                ft.Column(
                    controls=[
                        self.editButton, self.deleteButton
                    ]
                )
            ]
        )
    
    def edit_member(self, e: ft.TapEvent):
        dlg = MemberEdit(self, self.memberlist.page)
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            if dlg.newStatus.value == None:
                dlg.newStatus.value = self.status
            elif dlg.newName.value == None:
                dlg.newName.value = self.name
            elif dlg.newAddress.value == None:
                dlg.newAddress.value = self.address
            elif dlg.newEmail.value == None:
                dlg.newEmail.value = self.email
            elif dlg.newPhone.value == None:
                dlg.newPhone.value = self.phone
            else:
                member_controller = MemberController(sqlite3.connect('src/db/library.db'))
                member_controller.editMember(self.id, dlg.newName.value, dlg.newAddress.value, dlg.newEmail.value, dlg.newPhone.value, dlg.newStatus.value, self.booksBorrowed)
                self.memberlist.update_data()
    
    def delete_member(self, e: ft.TapEvent):
        dlg = ConfirmDialog(self.memberlist.page, "delete member")
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            member_controller = MemberController(sqlite3.connect('src/db/library.db'))
            member_controller.removeMember(self.id)
            self.memberlist.update_data()

class MemberEdit():
    def __init__(self, member: ft.Container, page: ft.Page):
        self.member = member
        self.page = page
        self.doAction = False

        self.newName = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
            value=self.member.name
        )
        self.newNameRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Name: "), self.newName
            ]
        )
        self.newAddress = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
            value=self.member.address
        )
        self.newAddressRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Address: "), self.newAddress
            ]
        )
        self.newEmail = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
            value=self.member.email
        )
        self.newEmailRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Email: "), self.newEmail
            ]
        )
        self.newPhone = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
            value=self.member.phone
        )
        self.newPhoneRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Phone: "), self.newPhone
            ]
        )
        self.newStatus = ft.Dropdown(
            bgcolor=ft.colors.GREY_900,
            options=[
                ft.dropdown.Option("active"),
                ft.dropdown.Option("not active")
            ]
        )
        self.newStatusRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Status: "), self.newStatus
            ]
        )

        self.dlg = ft.AlertDialog(
            modal=False,
            title=ft.Text("Edit Member"),
            content=ft.Column(
                width=400,
                height=300,
                controls=[
                    ft.Text("ID: "+str(self.member.id)),
                    self.newNameRow,
                    self.newAddressRow,
                    self.newEmailRow,
                    self.newPhoneRow,
                    self.newStatusRow
                ]
            ),
            actions=[
                ft.TextButton("Cancel", on_click=self.close_dlg),
                ft.TextButton("Save", on_click=self.close_dlg_do),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            bgcolor=ft.colors.BLACK
        )

    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    
    def close_dlg_do(self, e: ft.TapEvent):
        self.doAction = True
        self.dlg.open = False
        self.page.update()
    
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()

class MemberAdd():
    def __init__(self, page: ft.Page):
        self.page = page
        self.doAction = False

        self.newName = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
        )
        self.newNameRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Name: "), self.newName
            ]
        )
        self.newAddress = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
        )
        self.newAddressRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Address: "), self.newAddress
            ]
        )
        self.newEmail = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
        )
        self.newEmailRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Email: "), self.newEmail
            ]
        )
        self.newPhone = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
        )
        self.newPhoneRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Phone: "), self.newPhone
            ]
        )
        self.newStatus = ft.Dropdown(
            bgcolor=ft.colors.GREY_900,
            options=[
                ft.dropdown.Option("active"),
                ft.dropdown.Option("not active")
            ]
        )
        self.newStatusRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Status: "), self.newStatus
            ]
        )

        self.dlg = ft.AlertDialog(
            modal=False,
            title=ft.Text("New Member"),
            content=ft.Column(
                width=400,
                height=200,
                controls=[
                    self.newNameRow,
                    self.newAddressRow,
                    self.newEmailRow,
                    self.newPhoneRow,
                    self.newStatusRow
                ]
            ),
            actions=[
                ft.TextButton("Cancel", on_click=self.close_dlg),
                ft.TextButton("Add", on_click=self.close_dlg_do),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            bgcolor=ft.colors.BLACK
        )

    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    
    def close_dlg_do(self, e: ft.TapEvent):
        self.doAction = True
        self.dlg.open = False
        self.page.update()
    
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()

class MemberList(ft.GridView):
    def __init__(self):
        super().__init__()
        self.child_aspect_ratio = 2.5
        self.runs_count = 2
    def update_data(self):
        member_controller = MemberController(sqlite3.connect('src/db/library.db'))
        self.spacing = 10
        self.controls = []
        for m in member_controller.displayMember():
            mc = MemberContainer(self, m[0], m[1], m[2], m[3], m[4], m[5], m[6])
            self.controls.append(mc)
        self.update()
    def filter_data(self, input):
        member_controller = MemberController(sqlite3.connect('src/db/library.db'))
        self.spacing = 10
        self.controls = []
        for m in member_controller.searchMember(input):
            mc = MemberContainer(self, m[0], m[1], m[2], m[3], m[4], m[5], m[6])
            self.controls.append(mc)
        self.update()

class MemberView(ft.View):
    def __init__(self, page:ft.Page):
        super().__init__()
        self.page = page
        self.route = "/members"
        self.scroll = True
        self.members = MemberList()
        self.search = SearchBar(self.members)
        self.add = ft.ElevatedButton(text="New", icon=ft.icons.ADD, bgcolor=ft.colors.GREEN, color=ft.colors.WHITE, on_click=self.add_member)
        self.controls=[
            ft.Column(
                horizontal_alignment=ft.CrossAxisAlignment.CENTER,
                expand=True,
                controls=[
                    NavBar(page),
                    ft.Text("MEMBER LIST", weight=ft.FontWeight.BOLD, size=22),
                    ft.Row(
                        alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
                        controls=[
                            self.search, self.add
                        ]
                    ),
                    self.members
                ]
            )
        ]
    def add_member(self, e: ft.TapEvent):
        dlg = MemberAdd(self.page)
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            member_controller = MemberController(sqlite3.connect('src/db/library.db'))
            if dlg.newName.value == None or dlg.newAddress.value == None or dlg.newEmail.value == None or dlg.newPhone.value == None or dlg.newStatus.value == None:
                err = ErrorDialog(self.page, "Insert failed. All fields must be filled.")
                err.open_dlg(e)
            else:
                member_controller.addMemberByFields(dlg.newName.value, dlg.newAddress.value, dlg.newEmail.value, dlg.newPhone.value, dlg.newStatus.value)
                self.members.update_data()