import flet as ft

# dummy = {
#     'book':[
#         {'title':'Title1', 'isbn':'ISBN1', 'cover':'nocover.png', 'status':'available', 'amount':3},
#         {'title':'Title2', 'isbn':'ISBN2', 'cover':'nocover.png', 'status':'not available', 'amount':1},
#         {'title':'Title3', 'isbn':'ISBN3', 'cover':'nocover.png', 'status':'available', 'amount':999},        
#     ],
#     'member':[
#         {'id':1, 'name':'Name1', 'address':'Address1', 'email':'one@email.com', 'phone':'111', 'status':'active'},
#         {'id':2, 'name':'Name2', 'address':'Address2', 'email':'two@email.com', 'phone':'222', 'status':'active'},
#         {'id':3, 'name':'Name3', 'address':'Address3', 'email':'three@email.com', 'phone':'333', 'status':'not active'},
#     ],
#     'loan':[
#         {'id':1, 'bookISBN':'ISBN1', 'memberID':'1', 'borrowDate':'kapan', 'returnDate':'kapan'}
#     ]
# }

class NavBar(ft.Container):
    def __init__(self, page: ft.Page):
        super().__init__()
        self.name = ft.Text("MyPerpus", weight=ft.FontWeight.BOLD, size=17)
        self.books = ft.TextButton("Books", on_click=lambda _: page.go("/books"),
                                   style=ft.ButtonStyle(color=ft.colors.BLUE_ACCENT))
        self.members = ft.TextButton("Members", on_click=lambda _: page.go("/members"),
                                     style=ft.ButtonStyle(color=ft.colors.BLUE_ACCENT))
        self.loans = ft.TextButton("Loans", on_click=lambda _: page.go("/loans"),
                                   style=ft.ButtonStyle(color=ft.colors.BLUE_ACCENT))

        self.content = ft.Row(
            controls=[ft.Container(expand=3, content=self.name),
            ft.Row(expand=1, controls=[self.books, self.members, self.loans])
            ]
        )
        self.padding = 10
        self.border = ft.Border(bottom=ft.BorderSide(width=1, color=ft.colors.WHITE))

class ConfirmDialog():
    def __init__(self, page: ft.Page, action: str):
        self.doAction = False
        self.page = page
        self.dlg = ft.AlertDialog(
            modal=False,
            content=ft.Text(f"Are you sure you want to {action}?"),
            actions=[
                ft.TextButton("Cancel", on_click=self.close_dlg),
                ft.TextButton(f"{action.capitalize()}", on_click=self.close_dlg_do)
            ],
            actions_alignment=ft.MainAxisAlignment.CENTER,
            bgcolor=ft.colors.BLACK
        )
    
    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    
    def close_dlg_do(self, e: ft.TapEvent):
        self.doAction = True
        self.dlg.open = False
        self.page.update()
    
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()

class SearchBar(ft.Container):
    def __init__(self, list: ft.ListView):
        super().__init__()
        self.list = list
        self.searchInput = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            width=800,
            height=30,
            text_size=15,
            content_padding=5,
            cursor_color=ft.colors.WHITE,
        )
        self.searchButton = ft.IconButton(
            icon=ft.icons.SEARCH,
            icon_color=ft.colors.WHITE,
            bgcolor=ft.colors.INDIGO_ACCENT,
            height=30,
            width=30,
            icon_size=15,
            on_click=self.search
        )
        self.content = ft.Row(
            alignment=ft.MainAxisAlignment.START,
            height=30,
            controls=[
                self.searchInput,
                self.searchButton
            ]
        )
    
    def search(self, e: ft.TapEvent):
        if self.searchInput.value != None:
            self.list.filter_data(self.searchInput.value)

class ErrorDialog():
    def __init__(self, page: ft.Page, message: str):
        self.page = page
        self.dlg = ft.AlertDialog(
            modal=False,
            title=ft.Text("Error"),
            content=ft.Text(message),
            actions=[
                ft.TextButton("OK", on_click=self.close_dlg)
            ],
            actions_alignment=ft.MainAxisAlignment.CENTER,
            bgcolor=ft.colors.BLACK
        )
    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()