import sqlite3

class Member:
    # constructor
    def __init__(self, ID : int, name : str, address : str, email : str, phone : str, status : str, booksBorrowed : int):
        self.__ID = ID
        self.__name = name
        self.__address = address
        self.__email = email
        self.__phone = phone
        self.__status = status
        self.__booksBorrowed = booksBorrowed
    
    # getter setter
    def getID(self) -> int: return self.__ID
    def getName(self) -> str: return self.__name
    def getAddress(self) -> str: return self.__address
    def getEmail(self) -> str: return self.__email
    def getPhone(self) -> str: return self.__phone
    def getStatus(self) -> str: return self.__status
    def getBooksBorrowed(self) -> int: return self.__booksBorrowed
    
    def setName(self, name : str) -> None: self.__name = name
    def setAddress(self, address : str) -> None: self.__address = address
    def setEmail(self, email : str) -> None: self.__email = email
    def setPhone(self, phone : str) -> None: self.__phone = phone
    def setStatus(self, status : str) -> None: self.__status = status
    def setBooksBorrowed(self, booksBorrowed : int) -> None: self.__booksBorrowed = booksBorrowed

class MemberController:
    # constructor
    def __init__(self, connection):
        self.connection = connection
        self.create_table()
    
    def create_table(self):
        cursor = self.connection.cursor()
        cursor.execute('''
            CREATE TABLE IF NOT EXISTS Member(
                ID INTEGER PRIMARY KEY AUTOINCREMENT,
                name TEXT NOT NULL,
                address TEXT NOT NULL,
                email TEXT NOT NULL,
                phone TEXT NOT NULL,
                status TEXT NOT NULL,
                booksBorrowed INTEGER NOT NULL
            )
        ''')
        self.connection.commit()
    
    def addMember(self, member : Member):
        cursor = self.connection.cursor()
        cursor.execute('''
            INSERT INTO Member (name, address, email, phone, status, booksBorrowed)
            VALUES (?, ?, ?, ?, ?, ?)
        ''', (member.getName(), member.getAddress(), member.getEmail(), member.getPhone(), member.getStatus(), member.getBooksBorrowed(),))
        self.connection.commit()

    def addMemberByFields(self, name, address, email, phone, status):
        cursor = self.connection.cursor()
        cursor.execute('''
            INSERT INTO Member (name, address, email, phone, status, booksBorrowed)
            VALUES (?, ?, ?, ?, ?, ?)
        ''', (name, address, email, phone, status, 0,))
        self.connection.commit()
    
    def removeMember(self, id_member):
        cursor = self.connection.cursor()
        cursor.execute('''
            DELETE FROM Member WHERE ID = ?
        ''', (id_member,))
        self.connection.commit()

    def editMember(self, ID : int, name : str, address : str, email : str, phone : str, status : str, booksBorrowed : int):
        cursor = self.connection.cursor()
        cursor.execute('''
            UPDATE Member SET name = ?, address = ?, email = ?, phone = ?, status = ?, booksBorrowed = ? WHERE ID = ?
        ''', (name, address, email, phone, status, booksBorrowed, ID,))
        self.connection.commit()
    
    def displayMember(self):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Member
        ''')
        return cursor.fetchall()

    def getMember(self, ID):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Member WHERE ID = ?
        ''', (ID,))
        return cursor.fetchone()
    
    def searchMember(self, input):
        wildc_input = "%"+input+"%"
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Member WHERE name LIKE ? OR CAST(ID AS VARCHAR) LIKE ?
        ''', (wildc_input,wildc_input,))
        return cursor.fetchall()